#! /usr/bin/env python

#######################################
# A script to calculate statistics from
# a text file using NumPy
# Author: <YOUR NAME>
# Email: <YOUR EMAIL>
# Date: DD/MM/YYYY
# Version: 1.0
#######################################

from math import *
import numpy as np
import scipy as sp
from scipy.stats import *
from pylab import *

class CalculateStatistics (object):
    
    def run(self):
        # Set up lists to hold output data
        beechDiameter = list()
        ashDiameter = list()
        birchDiameter = list()
        oakDiameter = list()
        sycamoreDiameter = list()
        otherDiameter = list()
        
        # Open input and output files
        inFileName = 'PenglaisWoodsData.csv'
        outFileName = 'PenglaisWoodsStats.csv'
        inFile = open(inFileName, 'r')
        outFile = open(outFileName,'w')
        
        # Loop through in file and read in data
        header = True 
        for eachLine in inFile:
            if header: # Skip header row
                print 'Skipping header row'
                header = False
            else:
                substrs = eachLine.split(',',eachLine.count(','))
                
                species = substrs[3]
                if substrs[4].isdigit: # Check diameter is a number 
                    diameter = float(substrs[4])
                
                    if species == 'BEECH':
                        beechDiameter.append(diameter)
                    elif species == 'ASH':
                        ashDiameter.append(diameter)
                    elif species == 'BIRCH':
                        birchDiameter.append(diameter)
                    elif species == 'OAK':
                        oakDiameter.append(diameter)
                    elif species == 'SYC':
                        sycamoreDiameter.append(diameter)
                    else:
                        otherDiameter.append(diameter)
        
        # Convert to NumPy arrays
        beechDiameter = np.array(beechDiameter)
        ashDiameter = np.array(ashDiameter)
        birchDiameter = np.array(birchDiameter)
        oakDiameter = np.array(oakDiameter)
        sycamoreDiameter = np.array(sycamoreDiameter)
        otherDiameter = np.array(otherDiameter)
        
        # Write header line to output file
        headerLine = 'species, meanDiameter, medianDiameter, stDevDiameter\n'
        outFile.write(headerLine)
    
        # Calculate statistics for each species and write to file
        outLine = 'Beech,' + self.createStatsLine(beechDiameter) + '\n'
        outFile.write(outLine)
        outLine = 'Ash,' + self.createStatsLine(ashDiameter) + '\n'
        outFile.write(outLine)
        outLine = 'Birch,' + self.createStatsLine(birchDiameter) + '\n'
        outFile.write(outLine)
        outLine = 'Oak,' + self.createStatsLine(oakDiameter) + '\n'
        outFile.write(outLine)
        outLine = 'Sycamore,' + self.createStatsLine(sycamoreDiameter) + '\n'
        outFile.write(outLine)
        outLine = 'Other,' + self.createStatsLine(otherDiameter) + '\n'
        outFile.write(outLine)
        
        print 'Statistics writen to: ' + outFileName
        
    
    def createStatsLine(self, inArray):
        # Calculate statsistics for array and return output line.
        meanArray = np.mean(inArray)
        medianArray = np.median(inArray)
        stDevArray = np.std(inArray)
        skewArray = skew(inArray)
        
        # Create output line with stats
        statsLine = str(meanArray) + ',' + str(medianArray) + ',' + str(stDevArray)
        return statsLine
        
if __name__ == '__main__':
    obj = CalculateStatistics()
    obj.run()
