#!/usr/bin/env python

import sys
# Import the python Argument parser
import argparse
# Import the scipy filters.
from scipy import ndimage
#Import the numpy library
import numpy
# Import the RIOS image reader
from rios.imagereader import ImageReader
# Import the RIOS image writer
from rios.imagewriter import ImageWriter

# Define the function to iterate through
# the image.
def applyMedianFilter(inputFile, outputFile, fSize):
    # Get half the filter size, overlap between blocks
    hSize = (fSize-1)/2
    # Create the image reader for the input file 
    # and set the overlap to be half the image
    # filter size.
    reader = ImageReader(inputFile, overlap=hSize)
    # Define the image writer but cannot create
    # until within the loop as this need the 
    # information within the info object.
    writer = None
    # Loop through all the image blocks within 
    # the reader.
    for (info, block) in reader:
        # Create an output block of 
        # the same size as the input
        out = numpy.zeros_like(block)
        # Iterate through the image bands
        for i in range(len(out)):
            # Use scipy to run a median filter
            # on the image band data. The image
            # bands are filtered in turn
            ndimage.median_filter(block[i],size=fSize,output=out[i])
        # If it is the first time through the loop
        # (i.e., writer has a value of None) then 
        # create the loop.
        if writer is None:
            # Create the writer for output image.
            writer = ImageWriter(outputFile, 
                                 info=info, 
                                 firstblock=out, 
                                 drivername='HFA')
        else:
            # If the writer is created write the
            # output block to the file.
            writer.write(out)
    # Close the writer and calculate
    # the image statistics.
    writer.close(calcStats=True)

# This is the first part of the script to 
# be executed.
if __name__ == '__main__':
    # Create the command line options 
    # parser.
    parser = argparse.ArgumentParser()
    # Define the argument for specifying the input file.
    parser.add_argument("-i", "--input", type=str, 
                        help="Specify the input image file.")
    # Define the argument for specifying the output file.
    parser.add_argument("-o", "--output", type=str, 
                        help="Specify the output image file.")
    # Define the argument for the size of the image filter.
    parser.add_argument("-s", "--size", default=3, type=int, 
                        help="Filter size.")
    # Call the parser to parse the arguments.
    args = parser.parse_args()
    
    # Check that the input parameter has been specified.
    if args.input == None:
        # Print an error message if not and exit.
        print "Error: No input image file provided."
        sys.exit()
        
    # Check that the output parameter has been specified.
    if args.output == None:
        # Print an error message if not and exit.
        print "Error: No output image file provided."
        sys.exit()
    
    # Call the function to execute a median filter
    # on the input image.
    applyMedianFilter(args.input, args.output, args.size)    
    