#!/usr/bin/env python

# Import the system library
import sys
# Import the RIOS rat library.
from rios import rat
# Import the python Argument parser
import argparse

# The applier function to multiply the input
# column by 2.
def multiplyRATCol(imageFile, inColName, outColName):
    # Read the input column
    col = rat.readColumn(imageFile, inColName)
    # Muliply the column by 2.
    col = col * 2
    # Write the output column to the file.
    rat.writeColumn(imageFile, outColName, col)

# This is the first part of the script to 
# be executed.
if __name__ == '__main__':
    # Create the command line options 
    # parser.
    parser = argparse.ArgumentParser()
    # Define the argument for specifying the input file.
    parser.add_argument("-i", "--input", type=str, 
                        help="Specify the input image file.")
    # Define the argument for specifying the input column name.
    parser.add_argument("-c", "--inname", type=str, 
                        help="Specify the input column name.")
    # Define the argument for specifying the output column name.
    parser.add_argument("-o", "--outname", type=str, 
                        help="Specify the output column name.")
    # Call the parser to parse the arguments.
    args = parser.parse_args()
    
    # Check that the input parameter has been specified.
    if args.input == None:
        # Print an error message if not and exit.
        print "Error: No input image file provided."
        sys.exit()
    # Check that the input parameter has been specified.
    if args.inname == None:
        # Print an error message if not and exit.
        print "Error: No input RAT column name provided."
        sys.exit()
    # Check that the input parameter has been specified.
    if args.outname == None:
        # Print an error message if not and exit.
        print "Error: No output RAT column name provided."
        sys.exit()
        
    # Otherwise, run the function to multiply the input column by 2.
    multiplyRATCol(args.input, args.inname, args.outname)
